unit Dbpost;

{********************************************************************}
{          DBPostEdit 1.0, a DBEdit that                             }
{          Checks UK postcodes.                                      }
{           Andrew Skinner 1998                                     }
{          Summerfield                                               }
{          Preston on the Hill                                       }
{          Warrington                                                }
{          WA4 4AZ      UK                                           }
{                                                                    }
{          E-mail me if you use this.                                }
{          Skinner_doc@compuserve.com                                }
{          Credit me please in finished apps.                        }
{                                                                    }
{          Otherwise free, distributed as seen                       }
{          Use totally at own risk                                   }
{                                                                    }
{          NB                                                        }
{          Will allow postcodes that do not exist                    }
{          but checks basic format, which is not                     }
{          possible with a simple edit mask                          }
{                                                                    }
{          Set TDBPostEdit.Checkcode to false to disable checking    }
{          This can only be set from code, not the object            }
{          inspector in this version <VBG>.                          }
{                                                                    }
{                                                                    }
{********************************************************************}

interface


uses

  WinProcs,
  Messages,
  Classes,
  Controls,
  Dialogs,
  DBCtrls,
  StdCtrls;

type
  TDbPostEdit = class(TDbEdit)
  private
  fCheckcode:boolean;
  { Private declarations }

  protected
  { Protected declarations }
  procedure KeyPress(var Key: Char); override;
  procedure CMExit(var Message: TCMExit); message CM_EXIT;
  Function IsUKPostCode(S:string):boolean;
  function LTrim(Const Str: String): String;
  function RTrim(Str: String): String;
  function Trim(Str: String):  String;
  function IsAlpha(Ch:char):boolean;
  function IsNum(Ch:char):boolean;

  public
  property Checkcode:boolean read fcheckcode write fcheckcode default True;
  { Public declarations }

  Constructor Create(AOwner:TComponent); override;
  published
  { Published declarations }

  end;

procedure Register;

Const
  Space = #$20;



implementation


Constructor TDbPostEdit.Create(AOwner:TComponent);
begin
 inherited create(AOwner);
 CharCase:=ecUpperCase;
 checkcode:=true;
 end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TDbPostEdit]);
end;

procedure TDbPostEdit.KeyPress(var Key: Char);
begin
  {limits input to vaild chars}
  if Not (Key in ['0'..'9','A'..'Z','a'..'z',#32,#8]) Then Key := #0;
  inherited KeyPress(Key);
end;

procedure TDbPostEdit.CMExit(var Message: TCMExit);
begin
if checkcode then
begin
{checks UK postcode,error if not ok, refocusses on control}
If not IsUkpostcode(text) then begin
MessageDlg('Not a valid UK postcode',mterror,[mbok],0);
self.setfocus;
end else
{calls usual exit procedure if ok}
text:=trim(text);
inherited;
end;
end;

Function TDBPostEdit.IsUKPostCode(S:string):boolean;
var
inok,outok,writingout:boolean;
incode,outcode:string;
spacecount,i:integer;

begin
{sets up variables and trims input}
inok:=false; outok:=false; writingout:=false;
incode:='';outcode:='';
S:=trim(S);spacecount:=0;

{checks for space in code}
for i:= 1 to length(S) do
begin
if s[i]= space then inc(spacecount);
end;

if spacecount=1 then
{continues processing if has a single space}
  begin
{splits into incode and outcode}
  for i:= 1 to length(S) do
  begin
    if s[i]= space then writingout:=true;
    if not writingout then incode:=incode+S[i] else if S[i] <> space then outcode:=outcode+S[i];
  end;

{code here to check incode and outcode are valid}
{outcode}
if (isnum(outcode[1])) and (isalpha(outcode[2])) and (isalpha(outcode[3])) then outok:=true;
if length(outcode)<> 3 then outok:=false;
{incode}
if length(incode)= 2 then begin
  if (isalpha(incode[1])) and (isnum(incode[2])) then inok:=true
  end else
if length(incode)= 3 then begin
  if (isalpha(incode[1])) and (isnum(incode[2]))and (isnum(incode[3])) then inok:=true;
  if (isalpha(incode[1])) and (isalpha(incode[2]))and (isnum(incode[3])) then inok:=true;
  if (isalpha(incode[1])) and (isnum(incode[2]))and (isalpha(incode[3])) then inok:=true;
  end else
if length(incode)= 4 then begin
  if (isalpha(incode[1])) and (isalpha(incode[2]))and (isnum(incode[3])) and (isnum(incode[4]))then inok:=true;
  if (isalpha(incode[1])) and (isalpha(incode[2]))and (isnum(incode[3])) and (isalpha(incode[4]))then inok:=true;
  end;
end;
if (spacecount = 1) and inok and outok then result:=true else result:=false;
{remove this line to forbid blanks,
but better to allow blanks and check with required field}
if length(S)=0 then result:=true;
end;




{string handling fuctions below, from TrimStr by Bob Swart}

  function TDbPostEdit.LTrim(Const Str: String): String;
  var len: Byte absolute Str;
      i: Integer;
  begin
    i := 1;
    while (i <= len) and (Str[i] = Space) do Inc(i);
    LTrim := Copy(Str,i,len)
  end {LTrim};

  function TDbPostEdit.RTrim(Str: String): String;
  var len: Byte absolute Str;
  begin
    while (Str[len] = Space) do Dec(len);
    RTrim := Str
  end {RTrim};

  function TDbPostEdit.Trim(Str: String): String;
  begin
    Trim := LTrim(RTrim(Str))
  end {Trim};

{char handling fuctions below, by ACS}

  function TDbPostEdit.IsAlpha(Ch:char):boolean;
  begin
  if (Ch in ['A'..'Z', 'a'..'z']) then Result := true else result:=false;
  {the bit 'a'..'z' is not needed in tDBPostEdit as Uppercase only, but
  it makes the function portable}
  end;


  function TDbPostEdit.IsNum(Ch:char):boolean;
  begin
  if (Ch in ['0'..'9']) then Result := true else result:=false;
  end;


end.
